@@ -1,6 +1,6 @@
 /*
 
-    Copyright (C) 2007,2008,2009,2010,2013 Alois Schloegl <alois.schloegl@gmail.com>
+    Copyright (C) 2007,2008,2009,2010,2013,2015 Alois Schloegl <alois.schloegl@ist.ac.at>
     This file is part of the "BioSig for C/C++" repository 
     (biosig4c++) at http://biosig.sf.net/ 
 
@@ -15,12 +15,14 @@
 #include <math.h>
 #include <stdlib.h>
 #include <string.h>
+#include <strings.h>
 
-#include "../biosig.h"
-extern int VERBOSE_LEVEL; 	// used for debugging, variable is always defined
+#include <biosig.h>
 
 #ifdef NDEBUG
 #define VERBOSE_LEVEL 0 	// turn off debugging information, but its only used without NDEBUG
+#else
+extern int VERBOSE_LEVEL; 	// used for debugging
 #endif
 
 
