@@ -42,7 +42,7 @@
 
 #include <errno.h>
 #include <math.h>
-#include <stdint.h>
+#include <inttypes.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
